/*

  []---------------------------------------------------------------------[]
  |                                FX.C                                   |
  |                                                                       |
  |         Avaliaao de expressoes numericas definidas por strings       |
  |                                                                       |
  |                 (C) 1995, Lenimar Nunes de Andrade                    |
  []---------------------------------------------------------------------[]

*/

#include <math.h>
#include "str.c"

int Erro = 0;
double Denominador_nulo = 1e-8;
char delimitadores[] = "+-*/^() !";

char *MsgErro[] = {
	/*  0 */ "Nenhum erro encontrado",
	/*  1 */ "Denominador nulo",
	/*  2 */ "Exponencial com base negativa",
	/*  3 */ "+, -, *, / , \^, ( ou ) esperado",
	/*  4 */ "Expressao mal iniciada",
	/*  5 */ "Caracter invalido na expressao",
	/*  6 */ "Quantidades de \"(\" e \")\" diferentes",
	/*  7 */ "A expressao contem alguma funcao nao definida",
	/*  8 */ "Memoria insuficiente",
	/*  9 */ "Erro na avaliacao da expressao",
	/* 10 */ "Operador seguido de outro operador",
	/* 11 */ "Expressao mal terminada"
      };

#define SEM_ERRO     0
#define DENOMI_NULO  1
#define BASE_NEGATI  2
#define OPER_ESPERA  3
#define MAL_INICIAD  4
#define CAR_INVALID  5
#define PARENT_DIFE  6
#define FUNCAO_DESC  7
#define FALTA_MEMOR  8
#define AVALIA_EXPR  9
#define OPER_JUNTOS 10
#define MAL_TERMINA 11

/* ----------------------------------------------------------------------- */

double Fatorial(double n) {

/* Calcula n! = n(n - 1)(n - 2) ... 3.2.1 */

  double prod = 1;
  int i;

  for(i = n; i >= 2; i -= 1) prod *= i;
  return prod;
}

/* ----------------------------------------------------------------------- */

int Corte(char *y, char *conj) {

/* Determina o local da separacao do string "y" com relacao 'as operacoes
   aritmeticas +, -, /, * ou ^ .  Retorna  -1  se houver erro guardando o
   numero do erro na variavel global Erro. */

  typedef struct {
    int posicao;
    char ch;
  } corte;

  corte *LocalDoCorte;
  unsigned n_operacoes;
  register int i, quant_provavel_corte, AbreParenteses, tam;
  int abriu, pos_corte;

  n_operacoes = ContaCaracteres(y, '+') + ContaCaracteres(y, '-') +
		ContaCaracteres(y, '*') + ContaCaracteres(y, '/') +
		ContaCaracteres(y, '^');
  if (n_operacoes == 0) return 0;

  LocalDoCorte = (corte *) malloc(n_operacoes*sizeof(corte));
  if (LocalDoCorte == NULL) { Erro = FALTA_MEMOR; return -1; }

  TAMANHO(y, tam);
  i = 0; quant_provavel_corte = 0; AbreParenteses = 0; abriu = FALSO;
  do {

    do {
      if (PERTENCE(y[i], conj) && !abriu && i > 0) {
        LocalDoCorte[quant_provavel_corte].posicao = i;
        LocalDoCorte[quant_provavel_corte++].ch = y[i];
      } else if (y[i] == '(') {
	abriu = VERDADEIRO;
	AbreParenteses++;
      } else if (y[i] == ')') AbreParenteses--;
      i++;
    } while (i < tam && (AbreParenteses || !abriu));

    abriu = FALSO;
  } while (i < tam );

  if (quant_provavel_corte == 0) { free(LocalDoCorte); return 0; }

  if (PERTENCE('+', conj))
    for (i = 0; i < quant_provavel_corte; i++)
      if (LocalDoCorte[i].ch == '+') {
        pos_corte = LocalDoCorte[i].posicao;
	free(LocalDoCorte);
	return pos_corte;
      }

  if (PERTENCE('-', conj))
    for (i = quant_provavel_corte - 1; i >= 0; i--)
      if (LocalDoCorte[i].ch == '-') {
	pos_corte = LocalDoCorte[i].posicao;
	free(LocalDoCorte);
	return pos_corte;
      }
  if (PERTENCE('*', conj))
    for (i = 0; i < quant_provavel_corte; i++)
      if (LocalDoCorte[i].ch == '*') {
	pos_corte = LocalDoCorte[i].posicao;
	free(LocalDoCorte);
	return pos_corte;
      }

  if (PERTENCE('/', conj))
    for (i = quant_provavel_corte - 1; i >= 0; i--)
      if (LocalDoCorte[i].ch == '/') {
	 pos_corte = LocalDoCorte[i].posicao;
	 free(LocalDoCorte);
	 return pos_corte;
       }

  if (PERTENCE('^', conj))
    for (i = 0; i < quant_provavel_corte; i++)
      if (LocalDoCorte[i].ch == '^') {
	pos_corte = LocalDoCorte[i].posicao;
	free(LocalDoCorte);
	return pos_corte;
      }

  Erro = AVALIA_EXPR;
  free(LocalDoCorte);
  return -1;
}

/* ----------------------------------------------------------------------- */

double Calcula_F(char *expressao) {

/* Avalia a expressao definida como string. Os espacos em branco devem ser
   eliminados antes. O string deve ser formado por letras maiusculas. */

  register int tam_expr, k, corte, parenteses_superfluos;
  double expr1, expr2;
  char *exp_esquerda, *exp_direita;

  /* Eliminacao de parenteses superfluos, por ex. como em ((sen((3))+4)) */
  parenteses_superfluos = (expressao[0] == '(');
  while (parenteses_superfluos) {
    TAMANHO(expressao, tam_expr);
    if (expressao[0] == '(' && AbreFechaParentese(expressao, 0) == tam_expr-1)
      strcpy(expressao, ExpressaoEntreParenteses(expressao, 0));
    else parenteses_superfluos = FALSO;
  }

  corte = Corte(expressao, "+/-*^");
  if (corte) {

    /* Expressao do tipo -a^b, que e' o mesmo que -(a^b) */
    if (expressao[corte] == '^' && expressao[0] == '-') {
      TAMANHO(expressao, tam_expr);
      for (k = 0; k < tam_expr; k++) expressao[k] = expressao[k + 1];
      return -(Calcula_F(expressao));
    }

    TAMANHO(expressao, tam_expr);
    exp_esquerda = (char *) malloc(sizeof(char)*(corte + 1));
    exp_direita = (char *) malloc(sizeof(char)*(tam_expr - corte));
    if (exp_esquerda == NULL || exp_direita == NULL) {
      Erro = FALTA_MEMOR;
      return 0;
    }
    for (k = 0; k < corte; k++) exp_esquerda[k] = expressao[k];
    exp_esquerda[k] = '\0';
    for (k = corte + 1; k < tam_expr; k++)
      exp_direita[k - corte - 1] = expressao[k];
    exp_direita[k - corte - 1] = '\0';

    expr1 = Calcula_F(exp_esquerda);
    free(exp_esquerda);
    expr2 = Calcula_F(exp_direita);
    free(exp_direita);

    switch (expressao[corte]) {
      case '+' : return expr1 + expr2;
      case '-' : return expr1 - expr2;
      case '*' : return expr1 * expr2;
      case '/' : { if (fabs(expr2) < Denominador_nulo) {
		     Erro = DENOMI_NULO;
		     return 0;
		   }
		   return expr1 / expr2;
		 }
      case '^' : { if (expr1 < 0) {
		     if (floor(expr2) == expr2) {
		       if (fmod(fabs(expr2), 2) == 1)
			 return -pow(-expr1, expr2);
		      else
			 return pow(-expr1, expr2);
		     } else {
		       Erro = BASE_NEGATI;
		       return 0;
		     }
		   }
		   return pow(expr1, expr2);
		 }
    }

  } else if (expressao[0] == '-') {
     TAMANHO(expressao, tam_expr);
     for (k = 0; k < tam_expr; k++) expressao[k] = expressao[k + 1];
     return -(Calcula_F(expressao));

  } else if (expressao[0] == '+') {
     TAMANHO(expressao, tam_expr);
     for (k = 0; k < tam_expr; k++) expressao[k] = expressao[k + 1];
     return Calcula_F(expressao);

  }  else if (COMPSTRN(expressao, "SQRT(", 5)) {
     ExpressaoEntreParenteses(expressao, 4);
     return sqrt(Calcula_F(expressao));

  }  else if (COMPSTRN(expressao, "EXP(", 4)) {
     ExpressaoEntreParenteses(expressao, 3);
     return exp(Calcula_F(expressao));

  }  else if (COMPSTRN(expressao, "LOG(", 4)) {
     ExpressaoEntreParenteses(expressao, 3);
     return log(Calcula_F(expressao));

  }  else if (COMPSTRN(expressao, "SQR(", 4)) {
     double x_aux;
     ExpressaoEntreParenteses(expressao, 4);
     x_aux = Calcula_F(expressao);
     return x_aux*x_aux;

  }  else if (COMPSTRN(expressao, "SIN(", 4)) {
     ExpressaoEntreParenteses(expressao, 3);
     return sin(Calcula_F(expressao));

  }  else if (COMPSTRN(expressao, "COS(", 4)) {
     ExpressaoEntreParenteses(expressao, 3);
     return cos(Calcula_F(expressao));

  }  else if (COMPSTRN(expressao, "TAN(", 4)) {
     ExpressaoEntreParenteses(expressao, 3);
     return tan(Calcula_F(expressao));

  }  else if (COMPSTRN(expressao, "ARCSIN(", 7)) {
     ExpressaoEntreParenteses(expressao, 6);
     return asin(Calcula_F(expressao));

  }  else if (COMPSTRN(expressao, "ARCCOS(", 7)) {
     ExpressaoEntreParenteses(expressao, 6);
     return acos(Calcula_F(expressao));

  }  else if (COMPSTRN(expressao, "ARCTAN(", 7)) {
     ExpressaoEntreParenteses(expressao, 6);
     return atan(Calcula_F(expressao));

  }  else if (COMPSTRN(expressao, "SINH(", 5)) {
     ExpressaoEntreParenteses(expressao, 4);
     return sinh(Calcula_F(expressao));

  }  else if (COMPSTRN(expressao, "COSH(", 5)) {
     ExpressaoEntreParenteses(expressao, 4);
     return cosh(Calcula_F(expressao));

  }  else if (COMPSTRN(expressao, "TANH(", 5)) {
     ExpressaoEntreParenteses(expressao, 4);
     return tanh(Calcula_F(expressao));

  }  else if (COMPSTRN(expressao, "FABS(", 5)) {
     ExpressaoEntreParenteses(expressao, 4);
     return fabs(Calcula_F(expressao));

  }  else if (COMPSTRN(expressao, "INT(", 4)) {
     ExpressaoEntreParenteses(expressao, 3);
     return floor(Calcula_F(expressao));

  }  else if (COMPSTRN(expressao, "ROUND(", 6)) {
     double x_aux, y_aux;
     ExpressaoEntreParenteses(expressao, 5);
     x_aux = Calcula_F(expressao);
     y_aux = floor(x_aux);
     if (x_aux - y_aux < 0.5) return y_aux; else return (y_aux + 1);

  }  else if (COMPSTRN(expressao, "SIGN(", 5)) {
     double x_aux;
     ExpressaoEntreParenteses(expressao, 4);
     x_aux = Calcula_F(expressao);
     if (x_aux >= 0)
       return 1;
     else
       return -1;

  } else {
    TAMANHO(expressao, tam_expr);
    if (expressao[tam_expr - 1] == '!') {
      expressao[tam_expr - 1] = '\0';
      return Fatorial(Calcula_F(expressao));
    }
  }

  return atof(expressao);
}

/* ----------------------------------------------------------------------- */

int CompilaExpr(char *x) {

/* Elimina espacos em branco, verifica se a sintaxe esta' correta e trans-
   forma x em maiusculas. Identifica expressoes como "sin" e "sen", "tan"
   e "tg", etc. Deve ser chamada antes de Calcula_F(). */

  register int i, tamanho, abre_parenteses = 0, fecha_parenteses = 0;
  char CaracterValido[] =
  "_! .,()^-/*+abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789",
    operadores[] = "!+-/*^)",
    algarismos[] = "0123456789.,",
    oper_algarismos[] = "!+-/*^_()0123456789.,";

  ELIMINAESPACOS(x);
  Maiusculas(x);
  TAMANHO(x, tamanho);

  if (PERTENCE(x[0], "!.,)^/*\x0")) {
    Erro = MAL_INICIAD;
    return FALSO;
  }
  if (PERTENCE(x[tamanho - 1], "+-^.,(/*")) {
    Erro = MAL_TERMINA;
    return FALSO;
  }
  for (i = 0; i < tamanho; i++) {
    if (!PERTENCE(x[i], CaracterValido)) {
      Erro = CAR_INVALID;
      return FALSO;
    }
    if (x[i] == ')' && x[i + 1] != '\0' && !PERTENCE(x[i + 1], operadores)) {
      Erro = OPER_ESPERA;
      return FALSO;
    }
    if (PERTENCE(x[i], algarismos) && x[i + 1] != '\0' &&
				      !PERTENCE(x[i + 1], oper_algarismos)) {
      Erro = OPER_ESPERA;
      return FALSO;
    }
    if (PERTENCE(x[i], "+-/*^") && x[i + 1] != '\0' &&
					    PERTENCE(x[i + 1], "+-/*^")) {
      Erro = OPER_JUNTOS;
      return FALSO;
    }
    if (x[i] == '(') abre_parenteses++;
    else if (x[i] == ')') fecha_parenteses++;
  }

  if (abre_parenteses != fecha_parenteses) {
    Erro = PARENT_DIFE;
    return FALSO;
  }

  SubstituiIDelimitado(x, "pi", "3.14159265358979323846", delimitadores);
  SubstituiIDelimitado(x, "ln", "LOG", delimitadores);
  SubstituiIDelimitado(x, "sen", "SIN", delimitadores);
  SubstituiIDelimitado(x, "tg", "TAN", delimitadores);
  SubstituiIDelimitado(x, "arcsen", "ARCSIN", delimitadores);
  SubstituiIDelimitado(x, "arctg", "ARCTAN", delimitadores);
  SubstituiIDelimitado(x, "senh", "SINH", delimitadores);
  SubstituiIDelimitado(x, "tgh", "TANH", delimitadores);
  SubstituiIDelimitado(x, "abs", "FABS", delimitadores);
  SubstituiIDelimitado(x, "sinal", "SIGN", delimitadores);
  SubstituiIDelimitado(x, "quad", "SQR", delimitadores);
  SubstituiChar(x, ',', '.');

  return VERDADEIRO;
}

/* ----------------------------------------------------------------------- */

#define MAX_CAR_EXP 300      /* Quantidade maxima de caracteres no string */
#define MAX_CAR_VAR  50      /* Tamanho maximo dos nomes das variaveis    */
char ExpressaoNumericaStr1[MAX_CAR_EXP], ExpressaoNumericaStr2[MAX_CAR_EXP],
     ExpressaoNumericaStr3[MAX_CAR_EXP], 
     var1[MAX_CAR_VAR] = "u", var2[MAX_CAR_VAR] = "v";
							   

/* ----------------------------------------------------------------------- */

double f_str1(double x, double y) {

  char xstr[MAX_CAR_VAR], ystr[MAX_CAR_VAR], copia[MAX_CAR_EXP];

  strcpy(copia, ExpressaoNumericaStr1);
  sprintf(xstr, "(%f)", x);
  sprintf(ystr, "(%f)", y);
  SubstituiIDelimitado(copia, var1, xstr, delimitadores);
  SubstituiIDelimitado(copia, var2, ystr, delimitadores);
  return Calcula_F(copia);
}

/* ----------------------------------------------------------------------- */

double f_str2(double x, double y) {

  char xstr[MAX_CAR_VAR], ystr[MAX_CAR_VAR], copia[MAX_CAR_EXP];

  strcpy(copia, ExpressaoNumericaStr2);
  sprintf(xstr, "(%f)", x);
  sprintf(ystr, "(%f)", y);
  SubstituiIDelimitado(copia, var1, xstr, delimitadores);
  SubstituiIDelimitado(copia, var2, ystr, delimitadores);
  return Calcula_F(copia);
}

/* ----------------------------------------------------------------------- */

double f_str3(double x, double y) {

  char xstr[MAX_CAR_VAR], ystr[MAX_CAR_VAR], copia[MAX_CAR_EXP];

  strcpy(copia, ExpressaoNumericaStr3);
  sprintf(xstr, "(%f)", x);
  sprintf(ystr, "(%f)", y);
  SubstituiIDelimitado(copia, var1, xstr, delimitadores);
  SubstituiIDelimitado(copia, var2, ystr, delimitadores);
  return Calcula_F(copia);
}

/* ----------------------------------------------------------------------- */

/*** FIM DE FX.C ***/
