/* 

   []--------------------------------------------------------------------[]
   |                               GAUSS.C                                |
   |                                                                      |
   |          Calcula coeficientes das formas quadraticas e curvaturas    |
   |	           de uma superficie regular X(u, v) em um ponto q        |
   |                                                                      |
   |                    (C) 1998-99, Lenimar N. Andrade                   |
   []--------------------------------------------------------------------[]

*/

#include "fx.c" 

/* ----------------------------------------------------------------------- */

double (*f1)(double u, double v), (*f2)(double u, double v),
       (*f3)(double u, double v);

char xuv[3*MAX_CAR_EXP+20];

/* ----------------------------------------------------------------------- */

/* Derivada parcial segunda de f com relacao `a primeira variavel */
#define DIF2_1(f,a,b,h)  ((-2.0*f(a,b)+f(a-2.0*h,b)+f(a+2.0*h,b))/(4.0*h*h))

/* Derivada parcial segunda de f com relacao `a segunda variavel */
#define DIF2_2(f,a,b,h)  ((-2.0*f(a,b)+f(a,b-2.0*h)+f(a,b+2.0*h))/(4.0*h*h))

/* Derivada parcial segunda mista de f */
#define DIF2_3(f,a,b,h)  ((f(a+h,b+h)-f(a+h,b-h)-f(a-h,b+h)+\
                                                      f(a-h,b-h))/(4.0*h*h))

/* Derivada parcial de f com relacao `a primeira variavel */
#define DIF1(f,a,b,h)  ((f(a+h,b)-f(a-h,b))/(2.0*h))

/* Derivada parcial de f com relacao `a segunda variavel */
#define DIF2(f,a,b,h)  ((f(a,b+h)-f(a,b-h))/(2.0*h))

/* Variaveis globais */
double h_erro = 1e-6,      /* Usado no calculo aproximado das derivadas    */
       z_erro = 1e-6;      /* Se |x| < z_erro, entao x sera' considerado 0 */

/* Produto vetorial (v1, v2, v3) = (a1, a2, a3) x (b1, b2, b3) */
#define PROD_VET(v1,v2,v3,a1,a2,a3,b1,b2,b3)  v1=(a2)*(b3)-(a3)*(b2);\
                                v2=(a3)*(b1)-(a1)*(b3);v3=(a1)*(b2)-(a2)*(b1)

typedef struct { double x, y, z; } vetor;       /* Vetor (x, y, z) no R3   */
typedef struct { double u, v; } ponto_dominio;  /* Ponto (u, v) no dominio */

/* ----------------------------------------------------------------------- */

void LeValores(ponto_dominio *q) {

  float u0, v0;

  printf("\n======================================================================");
  printf("\n     Clculo de curvaturas gaussiana e mdia e direoes principais");
  printf("\n         de uma superfcie parametrizada regular X(u, v) em q\n");
  printf("\n        (C) 1998-1999, Lenimar N. Andrade, lenimar@mat.ufpb.br");
  printf("\n======================================================================\n"); 
  printf("\n%c Operadores vlidos:  +  -  *  /  ^\n", 254);
  printf("\n%c Funoes vlidas: SEN,  COS,  TG,  ARCSEN,  ARCCOS,  ARCTG,  LN,", 254);
  printf("\n                   EXP,  SENH,  COSH,  TGH,   INT,  ROUND,  SQRT,");
  printf("\n                   SQR,  ABS,  SIGN\n");
  printf("\n%c Sao consideradas idnticas as funoes SIN e SEN, TAN e TG, LOG e LN\n\n", 254);

  printf("\nX(u, v) = (f(u, v), g(u, v), h(u, v))   q = (u0, v0)\n\n");
  printf("f(u, v) = ");
  gets(ExpressaoNumericaStr1);
  printf("g(u, v) = ");
  gets(ExpressaoNumericaStr2);
  printf("h(u, v) = ");
  gets(ExpressaoNumericaStr3);
  printf("\nu0 = "); 
  scanf("%f", &u0);
  printf("v0 = ");
  scanf("%f", &v0);

  q->u = u0, q->v = v0;

  if (!CompilaExpr(ExpressaoNumericaStr1)) {     /* Verifica se ha' erro */
    printf("\nERRO: %s\n", MsgErro[Erro]);       /* na   compilacao   da */
    return;                                      /* expressao  string    */
  }

  if (!CompilaExpr(ExpressaoNumericaStr2)) {    
    printf("\nERRO: %s\n", MsgErro[Erro]);      
    return;                                     
  }
  if (!CompilaExpr(ExpressaoNumericaStr3)) {    
    printf("\nERRO: %s\n", MsgErro[Erro]);      
    return;                                     
  }

}

/* ----------------------------------------------------------------------- */

void main(void) {

  ponto_dominio q = {1, 1};
  double k1, k2, H, K, e, f, g, E, F, G, A, B, C;
  vetor Xu, Xv, Xuu, Xvv, Xuv, N;
  
  LeValores(&q);

  f1 = f_str1, f2 = f_str2, f3 = f_str3;   

  Xu.x = DIF1(f1, q.u, q.v, h_erro);
  Xu.y = DIF1(f2, q.u, q.v, h_erro);
  Xu.z = DIF1(f3, q.u, q.v, h_erro);

  Xv.x = DIF2(f1, q.u, q.v, h_erro);
  Xv.y = DIF2(f2, q.u, q.v, h_erro);
  Xv.z = DIF2(f3, q.u, q.v, h_erro);

  PROD_VET(N.x, N.y, N.z, Xu.x, Xu.y, Xu.z, Xv.x, Xv.y, Xv.z);

  Xuu.x = DIF2_1(f1, q.u, q.v, h_erro);
  Xuu.y = DIF2_1(f2, q.u, q.v, h_erro);
  Xuu.z = DIF2_1(f3, q.u, q.v, h_erro);

  Xvv.x = DIF2_2(f1, q.u, q.v, h_erro);
  Xvv.y = DIF2_2(f2, q.u, q.v, h_erro);
  Xvv.z = DIF2_2(f3, q.u, q.v, h_erro);

  Xuv.x = DIF2_3(f1, q.u, q.v, h_erro);
  Xuv.y = DIF2_3(f2, q.u, q.v, h_erro);
  Xuv.z = DIF2_3(f3, q.u, q.v, h_erro);

  E = Xu.x*Xu.x + Xu.y*Xu.y + Xu.z*Xu.z;
  F = Xu.x*Xv.x + Xu.y*Xv.y + Xu.z*Xv.z;
  G = Xv.x*Xv.x + Xv.y*Xv.y + Xv.z*Xv.z;

  e = Xuu.x*N.x + Xuu.y*N.y + Xuu.z*N.z;
  f = Xuv.x*N.x + Xuv.y*N.y + Xuv.z*N.z;
  g = Xvv.x*N.x + Xvv.y*N.y + Xvv.z*N.z;

  K = (e*g - f*f)/(E*G - F*F);
  H = 0.5*(e*G - 2*f*F + E*g)/(E*G - F*F);

  k1 = H - sqrt(H*H - K);
  k2 = H + sqrt(H*H - K);

  sprintf(xuv, "F(u, v) = (%s, %s, %s)", ExpressaoNumericaStr1,
                          ExpressaoNumericaStr2, ExpressaoNumericaStr3);

  printf("\n\n%s\n", xuv);

  printf("\nq = (%f, %f)\n", q.u, q.v);

  printf("\nXu = (%f, %f, %f)  Xv = (%f, %f, %f)\n",
  Xu.x, Xu.y, Xu.z, Xv.x, Xv.y, Xv.z);

  printf("\nN = (%f, %f, %f)  Xuu = (%f, %f, %f)\n",
  N.x, N.y, N.z, Xuu.x, Xuu.y, Xuu.z);

  printf("\nXvv = (%f, %f, %f)  Xuv = (%f, %f, %f)\n",
  Xvv.x, Xvv.y, Xvv.z, Xuv.x, Xuv.y, Xuv.z);

  printf("\nE = %f  F = %f   G = %f\n", E, F, G);

  printf("\ne = %f  f = %f   g = %f\n", e, f, g);

  printf("\nK = %f   H = %f   k1 = %f   k2 = %f\n", K, H, k1, k2);

  if (fabs(k1 - k2) < z_erro) printf("\nPonto umblico\n");

  if (K > z_erro) printf("\nPonto elptico\n");
  else if (K < -z_erro) printf("\nPonto hiperblico\n");
  else if (fabs(K) <= z_erro && fabs(H) > z_erro) printf("\nPonto parablico\n");
  else if (fabs(K) <= z_erro && fabs(H) <= z_erro) printf("\nPonto planar\n");

  A = e - k1*E, B = f - k1*F, C = g - k1*G;
  if (fabs(B) > z_erro) {
    printf("\nDireao principal w1 = (%f, %f, %f)= Xu %c %f Xv\n",
    Xu.x - A/B*Xv.x, Xu.y - A/B*Xv.y, Xu.z - A/B*Xv.z,
    ((-A/B > 0)? '+':'-'), fabs(A/B));
  }
  else if (fabs(A) > z_erro) {
    printf("\nDireao principal w1 = (%f, %f, %f)= Xv %c %f Xu\n",
    Xv.x - B/A*Xu.x, Xv.y - B/A*Xu.y, Xv.z - B/A*Xu.z,
      ((-B/A > 0)? '+':'-'), fabs(B/A));
  }
  else if (fabs(C) > z_erro) {
    printf("\nDireao principal w1 = (%f, %f, %f)= Xu %c %f Xv\n",
    Xu.x - B/C*Xv.x, Xu.y - B/C*Xv.y, Xu.z - B/C*Xv.z,
      ((-B/C > 0)? '+':'-'), fabs(B/C));
  }

  A = e - k2*E, B = f - k2*F, C = g - k2*G;
  if (fabs(B) > z_erro) {
    printf("\nDireao principal w2 = (%f, %f, %f) = Xu %c %f Xv\n",
    Xu.x - A/B*Xv.x, Xu.y - A/B*Xv.y, Xu.z - A/B*Xv.z,
    ((-A/B > 0)? '+':'-'), fabs(A/B));
  }
  else if (fabs(A) > z_erro) {
    printf("\nDireao principal w2 = (%f, %f, %f) = Xv %c %f Xu\n",
    Xv.x - B/A*Xu.x, Xv.y - B/A*Xu.y, Xv.z - B/A*Xu.z,
  ((-B/A > 0)? '+':'-'), fabs(B/A));
  }
  else if (fabs(C) > z_erro) {
    printf("\nDireao principal w2 = (%f, %f, %f) = Xu %c %f Xv\n",
    Xu.x - B/C*Xv.x, Xu.y - B/C*Xv.y, Xu.z - B/C*Xv.z,
    ((-B/C > 0)? '+':'-'), fabs(B/C));
  }

}

/* ----------------------------------------------------------------------- */

/*** FIM DE "GAUSS.C" ***/
