/*

  []---------------------------------------------------------------------[]
  |                                 STR.C                                 |
  |                                                                       |
  |                  Arquivo para operaoes com strings                   |
  |                                                                       |
  |                 (C) 1995, Lenimar Nunes de Andrade                    |
  []---------------------------------------------------------------------[]

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

extern unsigned _stklen = 10000;

#define boolean  int
#define VERDADEIRO 1
#define FALSO      0

/*
			     =============
			      OBSERVAOES
			     =============

     Em todas as funoes deste arquivo:

   - o primeiro caracter de um string X e' o caracter X[0].

   - o parametro de retorno de uma funcao, se for um string, precisa ser
     suficientemente grande para conter a resposta.  O  tamanho  exigido
     no parametro de retorno deve poder ser calculado usando-se a funcao
     strlen() ou a macro TAMANHO() definida a seguir.
*/

/* ------------------------------------------------------------------------ */

/* Calcula o tamanho do string s. E' o mesmo que tam = strlen(s).
   Ex.: TAMANHO("teste", tam)   ---> tam = 5                       */

#define TAMANHO(s, tam)       tam = 0; if (s[0]) while(s[++tam])

/* ======================== ADICAO DE CARACTERES ========================== */

/* Acrescenta parenteses no inicio e no final de um string.
   Ex.: PARENTESES("x+y+z") = "(x+y+z)"            */

#define PARENTESES(s) AcrescentaCaracteres(s, '(', ')')

/* ------------------------------------------------------------------------ */

char *AcrescentaCaracteres(char *x, char caracter_inicial,
				    char caracter_final) {

/* Acrescenta caracteres no inicio e no fim do string x.
   Ex.: AcrescentaCaracteres("mensagem", '*', '!') = "*mensagem!"     */

  char *aux;
  register int i, j = 0, tam;

  TAMANHO(x, tam);
  if ((aux = (char *) malloc(sizeof(char)*(tam + 3))) == NULL) return x;

  aux[j++] = caracter_inicial;
  for (i = 0; i < tam; i++)
    aux[j++] = x[i];
  aux[j++] = caracter_final;
  aux[j] = '\0';
  strcpy(x, aux);
  free(aux);
  return x;
}

/* ------------------------------------------------------------------------ */

char *AdicionaADireita(char *s, int largura, char caracter) {

/* Adiciona caracteres `a direita de um string s em um campo de largura
   definida.
   Ex.: AdicionaADireita("Olezinho", 12, '+') = "Olezinho++++"
	AdicionaADireita("Olezinho", 20, '&') = "Olezinho&&&&&&&&&&&&"
	AdicionaADireita("Olezinho",  5, '&') = "Olezi"                 */

  register int tam = 0, j;

  TAMANHO(s, tam);

  if (tam < largura)
    for (j = 0; j < largura - tam; j++) s[tam + j] = caracter;

  s[largura] = '\0';
  return s;
}

/* ------------------------------------------------------------------------ */

char *AdicionaAEsquerda(char *s, int largura, char caracter) {

/* Adiciona caracteres `a esquerda de um string s em um campo de largura
   definida.
   Ex.: AdicionaAEsquerda("Olezinho", 12, '+') = "++++Olezinho"
	AdicionaAEsquerda("Olezinho", 20, '&') = "&&&&&&&&&&&&Olezinho"
	AdicionaAEsquerda("Olezinho",  5, '&') = "zinho"                */

  register int tam, j;

  TAMANHO(s, tam);

  if (tam <= largura) {
    for (j = largura - 1; j >= 0; j--)
      if (j >= largura - tam)
	s[j] = s[j - largura + tam];
      else
	s[j] = caracter;
  }
  else
    for (j = 0; j < largura; j++) s[j] = s[j + tam - largura];

  s[largura] = '\0';
  return s;
}

/* ====================== CONSTRUCAO DE STRINGS =========================== */

char *Copias(char *x, char *s, int n) {

/* Retorna n copias do string s. Se n < 1, e' retornado o proprio s.
   Ex.: Copias(x, "....|", 4) = "....|....|....|....|"           */

  int tam, i, j;

  TAMANHO(s, tam);
  strcpy(x, s);
  for (i = 1; i < n; i++)
    for (j = 0; j < tam; j++)
      x[tam*i + j] = s[j];
  if (n > 1) x[tam*n] = '\0';
  return x;
}

/* ------------------------------------------------------------------------ */

char *Caracteres(char *s, int inicio, int fim) {

/* Retorna um string formado pelos caracteres do intervalo [inicio, fim].
   Ex.: Caracteres(str, 'A', 'G') = "ABCDEFG"
	Caracteres(str, 'z', 't') = "zyxwvut"
	Caracteres(str, 176, 182) = ""   */

  register int i;

  if (inicio <= fim) {
    i = inicio;
    while (i <= fim) { s[i - inicio] = (char) i; i++;}
    s[i - inicio] = '\0';
  } else {
    i = inicio;
    while (i >= fim) { s[inicio - i] = (char) i; i--;}
    s[inicio - i] = '\0';
  }
  return s;
}

/* ------------------------------------------------------------------------ */

char *Repete(char *x, char caracter, int n) {

/* Retorna o string formado pelo caracter dado repetido n vezes.
   Ex.: Repete(x, '+', 6) = "++++++"  */

   register int i = 0;
   if (n < 0) n = -n;
   for (i = 0; i < n; i++) x[i] = caracter;
   x[i] = '\0';
   return x;
}

/* ======================= MODIFICACAO DE STRINGS ========================= */

char *Espacos(char *s, int n, char caracter) {

/* Retorna s com cada palavra separada por n caracteres. Espacos em branco
   no inicio ou no final do string sao eliminados. Se n < 0,  e' retornado
   o proprio s.
   Ex.: Espacos(" xx yy    zz", 2, '-') = "xx--yy--zz"
	Espacos(" xx yy    zz", 0, '.') = "xxyyzz"
	Espacos(" xx yy    zz", -2, ' ') = " xx yy    zz"
	Espacos(" xx yy    zz", 1, ' ') = "xx yy zz"            */

  int i = 0, j, k = 0, tam;
  char *copia, car_x = ' ';

  if (n < 0) return s;
  TAMANHO(s, tam);

  if ((copia = (char *) malloc(sizeof(char)*(tam + 1))) == NULL) return NULL;
  strcpy(copia, s);

  while (copia[i] == car_x) i++;
  while (copia[i] != car_x && copia[i])
    s[k++] = copia[i++];
  do {
    while (copia[i] == car_x) i++;
    if (copia[i] == '\0') break;
    for (j = 0; j < n; j++) s[k++] = caracter;
    while (copia[i] != car_x && copia[i])
      s[k++] = copia[i++];
  } while (copia[i]);

  free(copia);
  s[k] = '\0';
  return s;
}

/* ------------------------------------------------------------------------ */

char *Justifica(char *s, int largura) {

/* Distribui espacos  em  brancos entre as palavras de um string de modo
   que esse  string passe a  ter a largura especificada. Se o tamanho de
   s for  maior do que ou igual `a  largura, e' retornado  o  proprio s.
   Espacos em branco no inicio ou final nao sao considerados.
   Ex.: Justifica(" xx yy z", 10) = "xx  yy   z"  */

  int i = 0, j, k, quant_caracteres = 0, quant_palavras = 0, inicio, fim,
      espacos, n_palavras = 0, tam;
  char *copia, car_x = ' ';

  TAMANHO(s, tam);
  if (tam >= largura) return s;

  if ((copia = (char *) malloc(sizeof(char)*(largura + 1))) == NULL)
    return NULL;
  strcpy(copia, s);

  while (s[i]) {
    while (s[i] == car_x) i++;
    if (s[i]) {
      inicio = i;
      while (s[i] != car_x && s[i]) i++;
      fim = i - 1;
      quant_palavras++;
      quant_caracteres += fim - inicio + 1;
    }
  }

  if (quant_palavras > 1)
    espacos = (largura - quant_caracteres)/(quant_palavras - 1);
  else espacos = 0;

  i = k = 0;
  while (copia[i]) {
    while (copia[i] == car_x) i++;
    if (copia[i]) {
      inicio = i;
      while (copia[i] != car_x && copia[i]) i++;
      fim = i - 1;

      n_palavras++;
      if (n_palavras == 1) {
	for (j = inicio; j <= fim; j++) {
	  s[k] = copia[j];
	  k++;
	}
      } else if (n_palavras < quant_palavras) {
	for (j = 0; j < espacos; j++) {
	  s[k] = car_x;
	  k++;
	}
	for (j = inicio; j <= fim; j++) {
	  s[k] = copia[j];
	  k++;
	}
      } else {
	for (j = 0; j < largura - quant_caracteres -
					 (quant_palavras - 2)*espacos; j++) {
	  s[k] = car_x;
	  k++;
	}
	for (j = inicio; j <= fim; j++) {
	  s[k] = copia[j];
	  k++;
	}
      }
    }
  }
  free(copia);
  s[k] = '\0';
  return s;
}

/* ------------------------------------------------------------------------ */

char *Maiusculas(char *s) {

/* Converte o string s para letras maiusculas.
   Ex.: Maiusculas("Guizinha") = "GUIZINHA"      */

  int i = 0;
  while(s[i]) {
    if (s[i] >= 'a' && s[i] <= 'z') s[i] -= 32;
    i++;
  }
  return s;
}

/* ------------------------------------------------------------------------ */

char *Minusculas(char *s) {

/* Converte o string s para letras minusculas
   Ex.: Minusculas("DEBORA") = "debora"         */

  int i = 0;
  while(s[i]) {
    if (s[i] >= 'A' && s[i] <= 'Z') s[i] += 32;
    i++;
  }
  return s;
}

/* ------------------------------------------------------------------------ */

char *Sobrescreve(char *s1, char *s2, int posicao, char caracter) {

/* Insere sobrescrevendo o  string s2 em s1 na posicao indicada, preenchendo
   espacos com um caracter fornecido, se for necessario. Se posicao < 0,  os
   strings serao emendados. O  resultado da operacao e' devolvido em s1 (que
   precisa ser suficientemente  grande).  Alguns  caracteres de s1 podem ser
   sobrescritos.
   Ex.: Sobrescreve("America do Sul", "Central", 8, '+') = "America Central"
	Sobrescreve("Str", "ings", -1, '+') = "Strings"
	Sobrescreve("Teste", "final", 10, '+') = "Teste+++++final"  */

  int i, tam;

  TAMANHO(s1, tam);

  if (posicao < 0) posicao = tam;
  if (tam >= posicao) {
    i = 0;
    while (s2[i]) s1[i + posicao] = s2[i++];
    if (i + posicao > tam) s1[i + posicao] = '\0';
  } else {
    i = tam;
    while (i < posicao) {
      s1[i] = caracter;
      i++;
    }
    while (s2[i - posicao]) {
      s1[i] = s2[i - posicao];
      i++;
    }
    s1[i] = '\0';
  }
  return s1;
}

/* ------------------------------------------------------------------------ */

char *Insere(char *s1, char *s2, int posicao, char caracter) {

/* Insere o string s2 em s1  na  posicao  indicada preenchendo espacos
   com um  caracter  fornecido, se  for necessario. Nenhum caracter de
   s1 e' apagado. Se posicao < 0, os strings serao emendados.
   O resultado da operacao e' devolvido em s1 (que precisa ser sufici-
   entemente grande).
   Ex.: Insere("------", "Insercao", 2, '.') = "--Insercao----"
	Insere("Teste", "Dois", -1, '.') = "TesteDois"
	Insere("Teste", "Dois", 10, '.') = "Teste.....Dois"  */

  int i, tam1, tam2;
  char *copia;

  TAMANHO(s1, tam1);
  if (posicao < 0) posicao = tam1;
  if (tam1 >= posicao) {
    if ((copia = (char *) malloc(sizeof(char)*(tam1 + 1))) == NULL)
      return NULL;
    strcpy(copia, s1);
    TAMANHO(s2, tam2);
    i = posicao;
    while (copia[i]) s1[i + tam2] = copia[i++];
    i = 0;
    while (s2[i]) s1[i + posicao] = s2[i++];
    s1[tam1 + tam2] = '\0';
    free(copia);
  } else {
    i = tam1;
    while (i < posicao) {
      s1[i] = caracter;
      i++;
    }
    while (s2[i - posicao]) {
      s1[i] = s2[i - posicao];
      i++;
    }
    s1[i] = '\0';
  }

  return s1;
}

/* ------------------------------------------------------------------------ */

char *Reverso(char *s) {

/* Retorna o string s escrito ao contrario.
   Ex.: Reverso("Guizinha") = "ahniziuG"      */

   register int i = 0, j, tam;
   char x;

   TAMANHO(s, tam);
   j = tam - 1;
   while (i < j) {
     x = s[i];
     s[i] = s[j];
     s[j] = x;
     i++;
     j--;
   }
   s[tam] = '\0';
   return s;
}

/* ------------------------------------------------------------------------ */

char *Centraliza(char *s, int largura, char caracter) {

/* Centraliza um string em um campo de largura especificada. O campo
   e' preenchido usando o caracter especificado.
   Ex.: Centraliza("Guizinha", 12, '-') = "--Guizinha--"
	Centraliza("Guizinha", 15, '*') = "---Guizinha----"
	Centraliza("Guizinha", 3, '*')  = "Gui"              */

  register int tam, i, k;

  TAMANHO(s, tam);

  if (tam < largura) {
    i = (largura - tam)/2;
    for (k = largura - 1; k >= 0; k--)
      if (k >= i + tam || k < i)
	s[k] = caracter;
      else
	s[k] = s[k - i];
  }
  s[largura] = '\0';
  return s;
}

/* ================= RETIRADA DE CARACTERES OU SUBSTRINGS ================= */

/* Elimina espacos em branco de um string.
   Ex.: ELIMINAESPACOS(" A B C ") = "ABC"       */

#define ELIMINAESPACOS(s) EliminaCaracter(s, ' ')

/* ------------------------------------------------------------------------ */

char *EliminaCaracter(char *x, char car_x) {

/* Elimina todas as ocorrencias do caracter dado do string x.
   Ex.: EliminaEspacos(".1..2..3.", '.') = "123"    */

  register int i, j = 0, comprimento;
  char *aux;

  TAMANHO(x, comprimento);
  if ((aux = (char *) malloc(sizeof(char)*(comprimento + 1))) == NULL)
    return x;

  for (i = 0; i < comprimento; i++)
    if (x[i] != car_x) aux[j++] = x[i];
  aux[j] = '\0';
  strcpy(x, aux);
  free(aux);
  return x;
}

/* ------------------------------------------------------------------------ */

char *CortaAEsquerda(char *s, int n) {

/* Elimina os n primeiros caracteres `a esquerda de um string.
   Ex.: CortaAEsquerda("Esquilinha", 3) = "uilinha" */

  register int i;

  i = n;
  while (s[i]) s[i - n] = s[i++];
  s[i - n] = '\0';
  return s;
}

/* ------------------------------------------------------------------------ */

char *CortaADireita(char *s, int n) {

/* Elimina os n ultimos caracteres de um string.
   Ex.: CortaADireita("Esquilinha", 3) = "Esquili"  */

  register int tam;

  if (n < 0) n = 0;
  TAMANHO(s, tam);
  if (n < tam) s[n] = '\0';
  return s;
}

/* ------------------------------------------------------------------------ */

char *Elimina(char *s, int inicio, int fim) {

/* Retorna o string s apos a retirada dos  caracteres  do  intervalo
   [inicio, fim]. inicio negativo e' transformado em 0  e  fim maior
   do que o tamanho do string e' reajustado para o tamanho do string.
   Ex.: Elimina("universidade", 3, 7) = "unidade"
	Elimina("linguagem", -2, 2) = "guagem"                      */

   register int i, tam;

   if (inicio < 0) inicio = 0;
   TAMANHO(s, tam);
   if (fim > tam) fim = tam;

   i = inicio;
   while (s[i]) s[i++] = s[i + fim - inicio + 1];
   s[i] = '\0';
   return s;
}

/* ------------------------------------------------------------------------ */

char *Retira(char *s, char caracter, int opcao) {

/* Retira caracteres consecutivos  `a direita (se opcao > 0) ou `a esquerda
   (opcao < 0) de um string s. Se opcao == 0,  sao  retirados  tanto os  da
   direita quanto os da  esquerda.  Os  caracteres sao  retirados  ate' que
   apareca outro caracter diferente do especificado.
   Ex.: Retira("    Marina   ",   ' ',  1)  =  "    Marina"
	Retira("    Marina   ",   ' ', -1)  =  "Marina   "
	Retira("    Marina   ",   ' ',  0)  =  "Marina"
	Retira("****Marina***",   '*',  0)  =  "Marina"
	Retira("....  Marina...", '.', -1)  =  "  Marina..."
	Retira("== ==Marina=== ", '=',  0)  =  " ==Marina=== "        */

  register int tam, i, j;

  TAMANHO(s, tam);
  if (opcao <= 0) {
    i = 0;
    while (s[i] == caracter) i++;
    for (j = 0; j < tam - i; j++) s[j] = s[j + i];
    tam -= i;
  }
  if (opcao >= 0) {
    i = tam - 1;
    while (s[i] == caracter) { s[i--] = '\0'; tam--;}
  }
  s[tam] = '\0';
  return s;
}

/* ------------------------------------------------------------------------ */

char *Substr(char *s, int inicio, int fim) {

/* Retorna um substring de um string s.
   Ex.: Substr("Joao Pessoa", 5, 8) = "Pess"    */

  register int i, tam;

  if (inicio > fim) return NULL;
  if (inicio < 0) inicio = 0;
  TAMANHO(s, tam);
  if (fim >= tam) fim = tam - 1;

  i = inicio;
  while (i <= fim) s[i - inicio] = s[i++];
  s[i - inicio] = '\0';
  return s;
}

/* ------------------------------------------------------------------------ */

char *SubPalavras(char *s, int m, int n, char caracter) {

/* Retorna n palavras do string s a partir da m-esima palavra. O caracter
   que separa as palavras deve ser fornecido. Se a m-esima palavra nao
   existir (por exemplo, se m <= 0 ou m muito grande) sera' considerado
   m = 1;
   Ex.: SubPalavras("um dois  tres quatro ", 2, 3, ' ') = "dois  tres quatro"
	SubPalavras("um dois  tres quatro ", 2, 0, ' ') = ""
	SubPalavras("um.dois..tres.quatro.", -2, 2, ' ') = "um.dois"
	SubPalavras("um.dois..tres.quatro.", 3, 7, '.') = "tres.quatro"   */

  int i = 0, k = 0, inicio0, inicio = -1, fim = -1,
      quant_palavras = 0, Inicio = 0, Fim = 0;

  if (n <= 0) {s[0] = '\0'; return s; }

  while (s[i]) {
    while (s[i] == caracter) i++;
    if (s[i] != '\0' && inicio < 0) inicio0 = i;
    if (s[i]) {
      inicio = i;
      while (s[i] != caracter && s[i]) i++;
      fim = i - 1;
      quant_palavras++;
    }
    if (quant_palavras == m) Inicio = inicio;
    if (quant_palavras == m + n - 1) break;
  }

  if (m <= 0 || m > quant_palavras) Inicio = inicio0;
  if (fim >= 0) Fim = fim;
  for (k = Inicio; k <= Fim; k++) s[k - Inicio] = s[k];
  s[k - Inicio] = '\0';
  return s;
}

/* ======================== COMPARACAO DE STRINGS ========================= */

/* Compara os strings s1 e s2. Retorna 1 se forem identicos ou 0 forem
   distintos.
   Ex.: COMPSTR("teste", "teste") = 1
	COMPSTR("teste", "TESTE") = 0                  */

#define COMPSTR(s1, s2)  (Compara(s1, s2) < 0)

/* ------------------------------------------------------------------------ */

int Compara(char *s1, char *s2) {

/* Compara os strings s1 e s2 e retorna a posicao dos primeiros caracteres
   que forem diferentes. Retorna -1 se os strings forem identicos.
   Ex.: Compara("um", "dois") = 0
	Compara("teste", "testE") = 4
	Compara("teste", "teste") = -1
	Compara("Alo", "Alo!!!") = 3     */

  register int i = 0;

  while (s1[i] != '\0' && s2[i] != '\0' && s1[i] == s2[i]) i++;
  if (s1[i] == '\0' && s2[i] == '\0') return -1;
  else return i;
}

/* ------------------------------------------------------------------------ */

/* Compara os n primeiros caracteres dos strings s1 e s2. Retorna 1 se
   forem identicos ou 0 forem distintos.
   Ex.: COMPSTRN("teste", "teste2...", 5) = 1
	COMPSTRN("teste", "Teste2...", 5) = 0                  */

#define COMPSTRN(s1, s2, n) (ComparaN(s1, s2, n) < 0)

/* ------------------------------------------------------------------------ */

int ComparaN(char *s1, char *s2, int n) {

/* Compara os n primeiros caracteres  dos strings s1 e s2 e retorna
   a posicao dos primeiros caracteres que forem diferentes. Retorna
   -1 se os strings coincidirem nos n primeiros caracteres.
   Ex.: ComparaN("um", "dois", 10) = 0
	ComparaN("teste", "testE", 4) = -1
	ComparaN("Alo", "Alo!!!", 5) = 3
	ComparaN("Alo", "Alo!!!", 3) = -1     */

  register int i = 0;

  while (i < n && s1[i] != '\0' && s2[i] != '\0' && s1[i] == s2[i]) i++;
  if (i == n || (s1[i] == '\0' && s2[i] == '\0')) return -1;
  else return i;
}

/* ------------------------------------------------------------------------ */

/* Compara os strings s1 e s2.  Retorna 1 se forem identicos  ou  0  forem
   distintos. Letras maiusculas sao consideradas identicas `as respectivas
   letras minusculas.
   Ex.: COMPSTRI("teste", "TESTE") = 1       */

#define COMPSTRI(s1, s2) (ComparaI(s1, s2) < 0)

/* ------------------------------------------------------------------------ */

int ComparaI(char *s1, char *s2) {

/* Compara os strings s1 e s2 e retorna a posicao dos primeiros caracteres
   que forem diferentes. Retorna -1 se os strings forem identicos.  Letras
   maiusculas sao consideradas identicas `as respectivas letras minusculas.
   Ex.: Compara("um", "dois") = 0
	Compara("teste", "TESTE") = 0
	Compara("teste", "teste") = -1
	Compara("Alo", "Alo!!!") = 3     */

  register int i = 0;

  while (s1[i] != '\0' && s2[i] != '\0' &&
				    toupper(s1[i]) == toupper(s2[i])) i++;
  if (s1[i] == '\0' && s2[i] == '\0') return -1;
  else return i;
}

/* ------------------------------------------------------------------------ */

/* Compara os n primeiros caracteres dos strings s1 e s2. Retorna 1 se
   forem identicos ou 0 forem distintos. Nao  e' feita distincao entre
   letras maiusculas e as respectivas letras minusculas.
   Ex.: COMPSTRNI("teste", "teste2...", 5) = 1
	COMPSTRNI("teste", "TESTE2...", 5) = 1                  */

#define COMPSTRNI(s1, s2, n) (ComparaNI(s1, s2, n) < 0)

/* ------------------------------------------------------------------------ */

int ComparaNI(char *s1, char *s2, int n) {

/* Compara os n primeiros caracteres dos strings  s1  e  s2 e retorna a
   posicao dos primeiros caracteres que forem diferentes. Retorna -1 se
   os strings coincidirem nos n primeiros caracteres. Letras maiusculas
   sao consideradas identicas `as respectivas letras minusculas.
   Ex.: ComparaN("um", "dois", 10) = 0
	ComparaN("teste", "TESTE", 4) = -1
	ComparaN("Alo", "Alo!!!", 5) = 3
	ComparaN("Alo", "Alo!!!", 3) = -1     */

  register int i = 0;

  while (i < n && s1[i] != '\0' && s2[i] != '\0' &&
				    toupper(s1[i]) == toupper(s2[i])) i++;
  if (i == n || (s1[i] == '\0' && s2[i] == '\0')) return -1;
  else return i;
}

/* =================== CONTAGEM OU POSICAO DE ELEMENTOS =================== */

int AbreFechaParentese(char *x, int posicao) {

/* Determina o fecha-parentese associado a um abre-parentese e vice-versa.
   Retorna -1 se houver erro.
   Ex.:  AbreFechaParentese("(1 + log(x)*(x + 2))", 0)   = 19
			     ^                  ^
	 AbreFechaParentese("(1 + log(x)*(x + 2))", 10)  =  8
				     ^ ^
	 AbreFechaParentese("(1 + log(x)*(x + 2)))", 20) = -1
						 ^                  */

  if (x[posicao] == '(') {

    register int abriu = 1, tam;

    TAMANHO(x, tam);
    while (posicao < tam && abriu) {
      posicao++;
      if (x[posicao] == '(') abriu++;
      else if (x[posicao] == ')') abriu--;
    }
    if (abriu) return -1; else return posicao;

  } else if (x[posicao] == ')') {

    register int fechou = 1;

    while (posicao > 0 && fechou) {
      posicao--;
      if (x[posicao] == ')') fechou++;
      else if (x[posicao] == '(') fechou--;
    }
    if (fechou) return -1; else return posicao;
  }

  return -1;
}

/* ------------------------------------------------------------------------ */

char *ExpressaoEntreParenteses(char *x, int posicao) {

/* Retorna a expressao dentro dos parenteses cuja posicao e' indicada.
   Retorna "" se houver erro.

   Ex.:  ExpressaoEntreParenteses("(1+log(x)*(x+2))", 0) = "1+log(x)*(x+2)"
				   ^              ^
	 ExpressaoEntreParenteses("(1+log(x)*(x+2))", 8)  =  "x"
					 ^ ^
	 ExpressaoEntreParenteses("(1+log(x)*(x+2)))", 16) = ""
						   ^             */
   int posicao_aux;

   if ((posicao_aux = AbreFechaParentese(x, posicao)) < 0) {
     x[0] = '\0';
     return x;
   }
   if (posicao_aux < posicao) return Substr(x, posicao_aux + 1, posicao - 1);
   return Substr(x, posicao + 1, posicao_aux - 1);
}

/* ------------------------------------------------------------------------ */

/* Retorna o ultimo caracter diferente de espaco em branco de um string
   Ex.: ULTIMOCARACTER("    Terra   ") = 'a'         */

#define ULTIMOCARACTER(s) UltimoCaracterDiferente(s, ' ')

/* ------------------------------------------------------------------------ */

char UltimoCaracterDiferente(char *s, char x) {

/* Retorna o ultimo caracter do string s diferente de x.
   Ex.: UltimoCaracterDiferente("----UFPB----", '-') = 'B' */

  register int i, tam;

  TAMANHO(s, tam);
  i = tam - 1;

  while (i > 0 && (s[i] == x || s[i] == '\n')) i--;
  return s[i];
}

/* ------------------------------------------------------------------------ */

/* Retorna o primeiro caracter diferente de espaco em branco de um string.
   Ex.: PRIMEIROCARACTER("   Brasil   ") = 'B' */

#define PRIMEIROCARACTER(s) PrimeiroCaracterDiferente(s, ' ');

/* ------------------------------------------------------------------------ */

char PrimeiroCaracterDiferente(char *s, char x) {

/* Retorna o primeiro caracter do string s diferente de x.
   Ex.: PrimeiroCaracterDiferente("----UFPB----", '-') = 'U' */

  register int i = 0;

  while (s[i] != '\0' && s[i] == x) i++;
  return s[i];
}

/* ------------------------------------------------------------------------ */

int NPalavras(char *s, char caracter) {

/* Conta a quantidade de palavras em um string delimitadas por determinado
   caracter.
   Ex.: NPalavras("Hoje e'  11/5/95,  terca-feira", ' ') = 4
	NPalavras("...Turbo...C...", '.') = 2                      */

  int i = 0, quant_palavras = 0;

  while (s[i]) {
    while (s[i] == caracter) i++;
    if (s[i]) {
      while (s[i] != caracter && s[i]) i++;
      quant_palavras++;
    }
  }

  return quant_palavras;
}

/* ------------------------------------------------------------------------ */

char *PalavraPos(char *s, char caracter, int n) {

/* Retorna a n-esima palavra de um string s delitimada por caracter dado.
   Ex.: PalavraPos("abc   xyz  w  ABC  ", ' ', 4) = "ABC"
	PalavraPos("abc   xyz  w  ABC  ", ' ', 6) = ""
	PalavraPos("---Isto e' ----- so' ----um teste", '-', 2) = " so' "  */

  int i = 0, j, k = 0, inicio, fim, quant_palavras = 0;

  while (s[i]) {
    while (s[i] == caracter) i++;
    if (s[i]) {
      inicio = i;
      while (s[i] != caracter && s[i]) i++;
      fim = i - 1;
      quant_palavras++;
    }
    if (quant_palavras == n) {
      for (j = inicio; j <= fim; j++) {
	s[k] = s[j];
	k++;
      }
    }
  }
  s[k] = '\0';
  return s;
}

/* ------------------------------------------------------------------------ */

int UltimaOcorrencia(char *substring, char *string, int posicao) {

/* Retorna a ultima ocorrencia de um substring em um string antes de uma
   certa posicao.  Se  posicao < 0, sera'  considerado  todo  o  string.
   Se o substring nao ocorrer dentro  do intervalo indicado sera' retor-
   nado o valor -1.
   Ex.: UltimaOcorrencia("ca", "Matematica Aplicada", -2) = 15
	UltimaOcorrencia("ca", "Matematica Aplicada", 17) = 15
	UltimaOcorrencia("ca", "Matematica Aplicada", 5) = -1
	UltimaOcorrencia("ca", "Matematica Aplicada", 8) = 8      */

  register int i, j;

  if (posicao < 0) {
    posicao = 0;
    while(string[posicao]) posicao++;
  }
  for (i = posicao; i >= 0;  i--)
    for (j = 0; !substring[j] || string[i + j] == substring[j]; j++)
      if (!substring[j]) return i;
  return -1;

}

/* ------------------------------------------------------------------------ */

/* Verifica se o substring s1 ocorre em s2.
   Ex.: STR_PERTENCE("ghi", "abcdefghij") = 1     */

#define STR_PERTENCE(s1, s2) (PrimeiraOcorrencia(s1, s2, 0) >= 0)

/* ------------------------------------------------------------------------ */

int PrimeiraOcorrencia(char *substring, char *string, int posicao) {

/* Retorna a primeira ocorrencia de um substring em um string a partir
   de uma certa posicao. Retorna -1 se o substring nao for encontrado.
   Ex.: PrimeiraOcorrencia("de", "Universidade Federal", 0) = 10
	PrimeiraOcorrencia("de", "Universidade Federal", 12) = 15
	PrimeiraOcorrencia("xy", "Universidade Federal", 0) = -1        */

  register int i, j;

  for (i = posicao; string[i]; i++)
    for (j = 0; !substring[j] || string[i + j] == substring[j]; j++)
      if (!substring[j]) return i;
  return -1;
}

/* ------------------------------------------------------------------------ */

int UltimaOcorrenciaI(char *substring, char *string, int posicao) {

/* Retorna a ultima ocorrencia de um substring em um string antes de uma
   certa posicao.  Se  posicao < 0, sera'  considerado  todo  o  string.
   Se o substring nao ocorrer dentro  do intervalo indicado sera' retor-
   nado o valor -1. Letras maiusculas sao consideradas identicas `as
   respectivas letras minusculas.
   Ex.: UltimaOcorrencia("CA", "Matematica Aplicada", -2) = 15
	UltimaOcorrencia("Ca", "Matematica Aplicada", 17) = 15
	UltimaOcorrencia("ca", "Matematica Aplicada", 5) = -1
	UltimaOcorrencia("cA", "Matematica Aplicada", 8) = 8      */

  register int i, j;

  if (posicao < 0) {
    posicao = 0;
    while(string[posicao]) posicao++;
  }
  for (i = posicao; i >= 0;  i--)
    for (j = 0; !substring[j] || toupper(string[i + j])
			      == toupper(substring[j]); j++)
      if (!substring[j]) return i;
  return -1;

}

/* ------------------------------------------------------------------------ */

/* Verifica se o substring s1 ocorre em s2. Letras maiusculas sao identifi-
   cadas com as respectivas letras minusculas.
   Ex.: STR_PERTENCE_I("GHI", "abcdefghij") = 1      */

#define STR_PERTENCE_I(s1, s2) (PrimeiraOcorrenciaI(s1, s2, 0) >= 0)

/* ------------------------------------------------------------------------ */

int PrimeiraOcorrenciaI(char *substring, char *string, int posicao) {

/* Retorna a primeira ocorrencia de um substring em um string a partir
   de uma certa posicao. Retorna -1 se o substring nao for encontrado.
   Letras maiusculas sao consideradas identicas `as respectivas letras
   minusculas.
   Ex.: PrimeiraOcorrencia("DE", "Universidade Federal", 0) = 10
	PrimeiraOcorrencia("DE", "Universidade Federal", 12) = 15
	PrimeiraOcorrencia("xy", "Universidade Federal", 0) = -1        */

  register int i, j;

  for (i = posicao; string[i]; i++)
    for (j = 0; !substring[j] || toupper(string[i + j])
			      == toupper(substring[j]); j++)
      if (!substring[j]) return i;
  return -1;
}

/* ------------------------------------------------------------------------ */

int QuantasPalavras(char *s, int pos, char caracter) {

/* Retorna a quantidade de palavras (separadas pelo caracter dado) existem
   no string s ate' a posicao pos fornecida.
   Ex.: QuantasPalavras("   Catole' do Rocha  ", 1,  ' ')  = 0
	QuantasPalavras("   Catole' do Rocha  ", 3,  ' ')  = 1
	QuantasPalavras("   Catole' do Rocha  ", 10, ' ')  = 1
	QuantasPalavras("   Catole' do Rocha  ", 16, ' ')  = 3
	QuantasPalavras("   Catole' do Rocha  ", 90, ' ')  = 3    */

  register int i = 0, quant_palavras = 0;

  while (s[i]) {
    while (s[i] == caracter)
      if (i == pos) return quant_palavras; else i++;
    if (s[i]) {
      while (s[i] != caracter && s[i])
	if (i == pos) return (quant_palavras+1); else i++;
      quant_palavras++;
    }
  }
  return quant_palavras;
}

/* ------------------------------------------------------------------------ */

int Pos(char x, char *s) {

/* Retorna a primeira posicao do caracter x no string s. Retorna -1 se
   x nao pertencer a s.
   Ex.: Pos('a', "Matematica") = 1           */

  register int i;

  for(i = 0; s[i]; i++)
    if (s[i] == x) return i;
  return -1;
}

/* ------------------------------------------------------------------------ */

int PosI(char x, char *s) {

/* Retorna a primeira posicao do caracter x no string s. Retorna -1 se
   x nao pertencer a s. Nao e' feita distincao entre letras minusculas
   e maiusculas.
   Ex.: Pos('A', "Matematica") = 1           */

  register int i;

  for(i = 0; s[i]; i++)
    if (toupper(s[i]) == toupper(x)) return i;
  return -1;
}

/* ------------------------------------------------------------------------ */

int PosUlt(char x, char *s) {

/* Retorna a ultima posicao do caracter x no string s. Retorna -1 se
   x nao pertencer a s.
   Ex.: PosUlt('a', "Matematica") = 9        */

  register int i, tam;

  TAMANHO(s, tam);
  for(i = tam; i >= 0; i--)
    if (s[i] == x) return i;
  return -1;
}

/* ------------------------------------------------------------------------ */

int PosN(char x, char *s, int n) {

/* Retorna a posicao do caracter x no string s situado a partir na n-esima
   posicao . Retorna -1 se x nao pertencer a x (apo's a n-esima posicao).
   Ex.: PosN('a', "Matematica", 3) = 5
	PosN('m', "Matematica", 6) = -1                                  */

  register int i, tam;

  TAMANHO(s, tam);
  for(i = n; i < tam; i++)
    if (s[i] == x) return i;
  return -1;
}

/* ------------------------------------------------------------------------ */

int ContaCaracteres(char *x, char ch) {

/* Conta quantas vezes o caractere ch aparece em x.
   Ex.: ContaCarateres("Matematica", 'a') = 3
	ContaCarateres("Matematica", 'A') = 0       */

  int cont = 0, i, tam;

  TAMANHO(x, tam);
  for (i = 0; i < tam; i++)
    if (x[i] == ch) cont++;
  return cont;
}

/* ------------------------------------------------------------------------ */

int ContaCaracteresI(char *x, char ch) {

/* Conta quantas vezes o caractere ch aparece em x nao fazendo distincao
   entre letras maiusculas e as respectivas letras minusculas.
   Ex.: ContaCarateresI("Matematica", 'A') = 3       */

  int cont = 0, i, tam;

  TAMANHO(x, tam);
  for (i = 0; i < tam; i++)
    if (toupper(x[i]) == toupper(ch)) cont++;
  return cont;
}

/* ------------------------------------------------------------------------ */

int ContaStr(char *x, char *a) {

/* Conta quantas vezes o string "a" aparece no string "x".
   Ex.: ContaStr("Universidade Federal", "de") = 2             */

  int aux = 0, coincide, i = 0, j, k, tam_a, tam_x;

  TAMANHO(x, tam_x);
  TAMANHO(a, tam_a);
  if (tam_x < tam_a) return 0;

  while (i < tam_x) {
    j = i;
    k = 0;
    coincide = 0;

    while ((k < tam_a) && (j < tam_x) && (a[k] == x[j])) {
      coincide++;
      k++;
      j++;
    }

    if (coincide == strlen(a)) aux++;
    i++;
  }
  return aux;
}

/* ------------------------------------------------------------------------ */

int ContaStrI(char *x, char *a) {

/* Conta quantas vezes o string  "a"  aparece no string  "x". Letras
   maiusculas sao identificadas com as respectivas letras minusculas.
   Ex.: ContaStrI("Universidade Federal", "DE") = 2             */

  int aux = 0, coincide, i = 0, j, k, tam_a, tam_x;

  TAMANHO(x, tam_x);
  TAMANHO(a, tam_a);
  if (tam_x < tam_a) return 0;

  while (i < tam_x) {
    j = i;
    k = 0; coincide = 0;

    while ((k < tam_a) && (j < tam_x) && (toupper(a[k]) == toupper(x[j]))) {
      coincide++;
      k++;
      j++;
    }

    if (coincide == strlen(a)) aux++;
    i++;
  }
  return aux;
}

/* ================== TROCA OU SUBSTITUICAO DE CARACTERES ================= */

char *Traduz(char *s, char *tabela_entrada, char *tabela_saida) {

/* Retorna s apos a troca de caracteres de s em tabela_entrada[] pelos
   respectivos  caracteres  em tabela_saida[]. As tabelas devem ser do
   mesmo tamanho.
   Ex.: Traduz("PROGRAM", "GXAR", "BQEL") =  "PROBLEM"
	Traduz("+-+-Teste%^%^", "+-%^", "....") = "....Teste...."  */

  int i = 0, j, tam1, tam2;

  TAMANHO(tabela_entrada, tam1);
  TAMANHO(tabela_saida, tam2);
  if (tam1 != tam2) return NULL;

  while (s[i]) {
    for (j = 0; j < tam1; j++)
      if (s[i] == tabela_entrada[j]) s[i] = tabela_saida[j];
    i++;
  }
  return s;
}

/* ------------------------------------------------------------------------ */

char *Substitui(char *x, char *a, char *b) {

/* Substitui "a" por "b" em "x" .
   Ex.: Substitui("abcxyz ABC", "abc", "***") = "***xyz ABC"    */

  int tam_x, tam_a, coincide, i = 0, j, k;
  char *aux;

  TAMANHO(x, tam_x);
  TAMANHO(a, tam_a);
  if (tam_x < tam_a) return x;
  if ((aux = (char *) malloc(sizeof(char)*(tam_x + 1))) == NULL) return x;

  aux[0] = '\0';
  while (i <= tam_x) {
    j = i;
    k = 0;
    coincide = 0;

    while ((k < tam_a) && (j < tam_x) && (a[k] == x[j])) {
      coincide = coincide + 1;
      k++;
      j++;
    }

    if (coincide == tam_a) {
      strcat(aux, b);
      i += tam_a;
    } else {
      int n;
      n = strlen(aux);
      aux[n++] = x[i]; aux[n] = '\0';
      i++;
    }
  }

  strcpy(x, aux);
  free(aux);
  return x;
}

/* ------------------------------------------------------------------------ */

char *SubstituiI(char *x, char *a, char *b) {

/* Substitui "a" por "b" em "x" . Letras maiusculas sao identificadas
   com as respectivas letras minusculas.
   Ex.: SubstituiI("abcxyz ABC", "abc", "***") = "***xyz ***"    */

  int tam_x, tam_a, coincide, i = 0, j, k;
  char *aux;

  TAMANHO(a, tam_a);
  TAMANHO(x, tam_x);
  if (tam_x < tam_a) return x;
  if ((aux = (char *) malloc(sizeof(char)*(tam_x + 1))) == NULL) return x;

  aux[0] = '\0';
  while (i <= tam_x) {
    j = i;
    k = 0; coincide = 0;

    while ((k < tam_a) && (j < tam_x) && (toupper(a[k]) == toupper(x[j]))){
      coincide = coincide + 1;
      k++;
      j++;
    }

    if (coincide == tam_a) {
      strcat(aux, b);
      i += tam_a;
    } else {
      int n;
      n = strlen(aux);
      aux[n++] = x[i]; aux[n] = '\0';
      i++;
    }
  }

  strcpy(x, aux);
  free(aux);
  return x;
}

/* ------------------------------------------------------------------------ */

char *SubstituiDelimitado(char *x, char *a, char *b, char *limites) {

/* Substitui "a" por "b" em "x", desde que "a" esteja delimitado pelos
   caracteres limites.
   Ex.: SubstituiDelimitado("abcxyz ABC", "abc", "***", "c ") = "abcxyz ABC"
	SubstituiDelimitado("abcxyz ABC", "xyz", "***", "c ") = "abc*** ABC"     */

  int tam_x, tam_a, coincide, i = 0, j, k, lim, tam_lim, delimitado;
  char *aux;

  TAMANHO(x, tam_x);
  TAMANHO(a, tam_a);
  TAMANHO(limites, tam_lim);
  if (tam_x < tam_a) return x;
  if ((aux = (char *) malloc(sizeof(char)*(tam_x + 1))) == NULL) return x;

  aux[0] = '\0';
  while (i <= tam_x) {
    j = i;
    k = 0;
    coincide = 0;

    delimitado = FALSO;
    if (j > 1) {
      for (lim = 0; lim < tam_lim; lim++)
	if (limites[lim] == x[j - 1]) { delimitado = VERDADEIRO; break; }
    } else delimitado = VERDADEIRO;

    if (delimitado)
      while ((k < tam_a) && (j < tam_x) && (a[k] == x[j])) {
	coincide = coincide + 1;
	k++;
	j++;
      }

    delimitado = FALSO;
    if (j < tam_x - 1) {
      for (lim = 0; lim < tam_lim; lim++)
	if (limites[lim] == x[j]) { delimitado = VERDADEIRO; break; }
    } else delimitado = VERDADEIRO;

    if (delimitado && coincide == tam_a) {
      strcat(aux, b);
      i += tam_a;
    } else {
      int n;
      n = strlen(aux);
      aux[n++] = x[i]; aux[n] = '\0';
      i++;
    }
  }

  strcpy(x, aux);
  free(aux);
  return x;
}

/* ------------------------------------------------------------------------ */

char *SubstituiIDelimitado(char *x, char *a, char *b, char *limites) {

/* Semelhante a SubstituiDelimitado() identificando as letras maiusculas
   com as respectivas letras minusculas. */

  int tam_x, tam_a, coincide, i = 0, j, k, lim, tam_lim, delimitado;
  char *aux;

  TAMANHO(a, tam_a);
  TAMANHO(x, tam_x);
  TAMANHO(limites, tam_lim);
  if (tam_x < tam_a) return x;
  if ((aux = (char *) malloc(sizeof(char)*(tam_x + 1))) == NULL) return x;

  aux[0] = '\0';
  while (i <= tam_x) {
    j = i;
    k = 0; coincide = 0;

    delimitado = FALSO;
    if (j > 1) {
      for (lim = 0; lim < tam_lim; lim++)
	if (limites[lim] == x[j - 1]) { delimitado = VERDADEIRO; break; }
    } else delimitado = VERDADEIRO;

    if (delimitado)
      while ((k < tam_a) && (j < tam_x) && (toupper(a[k]) == toupper(x[j]))){
	coincide = coincide + 1;
	k++;
	j++;
      }

    delimitado = FALSO;
    if (j < tam_x - 1) {
      for (lim = 0; lim < tam_lim; lim++)
	if (limites[lim] == x[j]) { delimitado = VERDADEIRO; break; }
    } else delimitado = VERDADEIRO;

    if (delimitado && coincide == tam_a) {
      strcat(aux, b);
      i += tam_a;
    } else {
      int n;
      n = strlen(aux);
      aux[n++] = x[i]; aux[n] = '\0';
      i++;
    }
  }

  strcpy(x, aux);
  free(aux);
  return x;
}

/* ------------------------------------------------------------------------ */

char *SubstituiChar(char *x, char a, char b) {

/* Substitui 'a' por 'b' em "x" .
   Ex.: SubstituiChar("abcxyz ABC", 'x', '*') = "abc*yz ABC"    */

  register int i = 0;

  while (x[i] != '\0') {
    if (x[i] == a) x[i] = b;
    i++;
  }
  return x;
}

/* ====================== OPERACOES COM CONJUNTOS ========================= */

/* As funcoes a seguir interpretam um string como se  fosse um conjunto
   formado pelos caracteres do strings. Sao implementadas as principais
   operacoes com conjuntos.

   Ex.: A operacao  X = {1, 2, 3, 4}  {2, 4, 5}  pode  ser efetuada na
	forma Intersecao("1234", "245", X).                                */

/* ------------------------------------------------------------------------ */

/* Verifica se o elemento x pertence ao conjunto Y.
   Ex.: PERTENCE('1', "1234") = 1
	PERTENCE('5', "1234") = 0   */

#define PERTENCE(x, Y)  (Pos(x, Y) >= 0)

/* ------------------------------------------------------------------------ */

/* Verifica se o elemento x pertence ao conjunto Y sem fazer distincao
   entre letras minusculas e maiusculas.
   Ex.: PERTENCE_I('a', "ABCD") = 1
	PERTENCE_I('A', "12CD") = 0   */

#define PERTENCE_I(x, Y)  (PosI(x, Y) >= 0)

/* ------------------------------------------------------------------------ */

char *AnexaCaracter(char *s, char x) {

/* Anexa um caracter ainda nao pertencente a um string.
   Ex.: AnexaCaracter("abc", 'd') = "abcd"
	AnexaCaracter("abc", 'a') = "abc"                 */

  register int i;
  int tam;
  boolean novo = VERDADEIRO;

  TAMANHO(s, tam);
  for (i = 0; i < tam; i++)
    if (s[i] == x) novo = FALSO;
  if (novo) {
    s[tam] = x;
    s[tam + 1] = '\0';
  }
  return s;
}

/* ------------------------------------------------------------------------ */

/* Verifica se os conjuntos X e Y sao iguais
   Ex.: CONJ_IGUAIS("abcd", "dcabca") = 1
	CONJ_IGUAIS("123",  "234")    = 0          */

#define CONJ_IGUAIS(X, Y)   (EstaContido(X, Y) && EstaContido(Y, X))

/* ------------------------------------------------------------------------ */

boolean EstaContido(char *x, char *y) {

/* Verifica se o conjunto x esta' contido no conjunto y.
   Ex.: EstaContido("abc", "xy abx zw")  = 0
	EstaContido("abc", "ab xyz cd")  = 1 */

  register int i, tam;

  TAMANHO(x, tam);

  for (i = 0; i < tam; i++)
    if (Pos(x[i], y) < 0) return FALSO;

  return VERDADEIRO;
}

/* ------------------------------------------------------------------------ */

char *Intersecao(char *s1, char *s2, char *intersecao) {

/* Devolve o string dos caracteres que pertencem simultaneamente a s1
   e a s2.
   Ex.: Intersecao("ABCD", "ABXYZ", s) = "AB" */

  int i, tam;

  intersecao[0]  = '\0';
  TAMANHO(s1, tam);
  for (i = 0; i < tam; i++)
    if (PERTENCE(s1[i], s2)) AnexaCaracter(intersecao, s1[i]);
  return intersecao;
}

/* ------------------------------------------------------------------------ */

char *Uniao(char *s1, char *s2, char *uniao) {

/* Devolve o string formado pelos caracteres de  s1  ou de  s2,  sem
   caracteres repetidos.
   Ex.: Uniao("ABCD", "ABXYZ", s) = "ABCDXYZ" */

  int i, tam1, tam2;

  uniao[0]  = '\0';
  TAMANHO(s1, tam1);
  TAMANHO(s2, tam2);
  for (i = 0; i < tam1; i++) AnexaCaracter(uniao, s1[i]);
  for (i = 0; i < tam2; i++) AnexaCaracter(uniao, s2[i]);
  return uniao;
}

/* ------------------------------------------------------------------------ */

char *Diferenca(char *s1, char *s2, char *diferenca) {

/* Calcula a  diferenca  entre os conjuntos de caracterees s1 e s2,
  ou seja, devolve o string  formado pelos caracteres de s1 que nao
  estao fazem parte de s2.
  Ex.: Diferenca("1234abcd", "ab") = "1234cd"
       Diferenca("1234abcd", "xyz1994") = "23abcd"
       Diferenca("1234abcd", "56ef") = "1234abcd"       */

  int i, tam;

  diferenca[0]  = '\0';
  TAMANHO(s1, tam);
  for (i = 0; i < tam; i++)
    if (Pos(s1[i], s2) < 0) AnexaCaracter(diferenca, s1[i]);
  return diferenca;
}

/* ------------------------------------------------------------------------ */

char *EliminaRepeticao(char *s) {

/* Elimina os caracteres repetidos de um string s.
   Ex.: EliminaRepeticao("aa1 a1b1ab2b ab") = "a1 b2" */

  int i, tam;
  char *s_aux;

  TAMANHO(s, tam);
  if ((s_aux = (char *) malloc(sizeof(char)*(tam + 2))) == NULL) return s;
  s_aux[0] = '\0';

  for (i = 0; i < tam; i++)
    AnexaCaracter(s_aux, s[i]);

  strcpy(s, s_aux);
  free(s_aux);
  return s;
}

/* ------------------------------------------------------------------------ */

/*** FIM DE STR.C ***/
